﻿#include "EqPreCompile.h" //QApplication

#include "ServicePublisher.h"

/*!
 * \brief EqNetworkThread::~EqNetworkThread 析构函数。
 */
EqNetworkThread::~EqNetworkThread()
{
    quit (); //退出线程。

} //EqNetworkThread::~EqNetworkThread()

/*!
 * \brief GameInfo::GameInfo 默认构造函数。
 */
EqNetworkThread::EqNetworkThread()
{
    initializeMemberObjects(); //初始化成员对象。
    connectSignals(); //连接信号槽。

} //ServicePublisher::ServicePublisher()

/*!
 * \brief ServicePublisher::publishService 发布服务。
 * \param LanServiceName 服务名字。
 * \param LanServicePort 服务端口。
 * \param LanServiceProtocolType 服务协议类型。
 */
void EqNetworkThread::publishService(QString LanServiceName,quint16 LanServicePort,exist::ServicePublishMessage::ServiceProtocolType LanServiceProtocolType)
{
    Q_UNUSED(LanServiceName); //不使用这个参数。
    Q_UNUSED(LanServicePort); //不使用这个参数。
    Q_UNUSED(LanServiceProtocolType); //不使用这个参数。

    exist::ExistMessage existMessage; //消息。

    existMessage.set_messagetype (exist::ExistMessage::SERVICEPUBLISH); //设置消息类型。

    exist::ServicePublishMessage * servicePublishMessage=existMessage.mutable_servicepublishmessage (); //获取服务发布消息对象。

    servicePublishMessage->set_name (LanServiceName.toStdString ()); //设置服务名字。
    servicePublishMessage->set_port (LanServicePort); //设置端口号。
    servicePublishMessage->set_protocoltype (LanServiceProtocolType); //设置协议类型。

    string packageString=existMessage.SerializeAsString (); //序列化为字符串。

//    QByteArray datagram=QByteArray::fromStdString (packageString); //转换成字符数组。
    QByteArray datagram=QByteArray::fromRawData (packageString.data (),packageString.size ()); //转换成字符数组。

    qDebug() << __FILE__ << __LINE__ << __func__ << tr("Service name:") << LanServiceName << tr("Service port:") << LanServicePort << tr("Service protocol type:") << LanServiceProtocolType; //Debug.

    return;
} //void ServicePublisher::publishService(QString LanServiceName,quint16 LanServicePort,ServiceProtocolType LanServiceProtocolType)

/*!
 * \brief ServicePublisher::connectSignals 连接信号槽。
 */
void EqNetworkThread::connectSignals()
{
    qDebug() << __FILE__ << __LINE__ << __func__ ; //Debug.

    connect(&socketObject,&QUdpSocket::readyRead, this, &EqNetworkThread::readSocketData); //有数据包到达，则读取。
    connect(&receiveTimer,&QTimer::timeout,this,&EqNetworkThread::readSocketData); //定时器超时，则尝试从网络读取数据。
} //void ServicePublisher::connectSignals()

/*!
 * \brief EqNetworkThread::run 运行。
 */
void EqNetworkThread::run()
{
    exec(); //进入事件循环。
} //void EqNetworkThread::run()

/*!
 * \brief UdpServer::readSocketData 数据来了，则读取。
 */
void EqNetworkThread::readSocketData()
{
    char buffer[512];
    
    QHostAddress packageSender; //数据包发送者的地址。
    
    
    int nextDatagramSize=socketObject.pendingDatagramSize(); //获取下一个数据包的尺寸。
    quint16 senderPort; //发送者的端口号。
    
    
    int readDatagramSize=socketObject.readDatagram(buffer,nextDatagramSize,&packageSender,&senderPort); //读取数据包。
    
    if (readDatagramSize>0) //读入了有效的数据。
    {
        emit readedSocketData(buffer, readDatagramSize, packageSender); //发射信号，读取到了数据。
    } //if (n>0) //读入了有效的数据。
    
    
    if (socketObject.hasPendingDatagrams()) //还有数据包未读完。
    {
//            readSocketData(); //继续读取。
        } //if (socketObject.hasPendingDatagrams()) //还有数据包未读完。
} //void UdpServer::readSocketData()

/*!
 * \brief SBrowserWindow::initializeMemberObjects 初始化成员对象。
 */
void EqNetworkThread::initializeMemberObjects()
{
    //加入组播组：
    address= QHostAddress("239.173.40.5");
    
    
    bindAndSetSocketOptions(); //绑定并设置套接字选项。
    
    receiveTimer.setInterval (100); //0.1秒超时。
    receiveTimer.start(); //启动数据接收定时器。
    
    socketObject.setObjectName("socketObject"); //设置对象名字，用于调试。
    receiveTimer.setObjectName("receiveTimer"); //设置对象的名字，用于调试。
} //void SBrowserWindow::initializeMemberObjects()

/*!
 * \brief EqNetworkThread::bindAndSetSocketOptionsForWindows 针对Windows绑定及设置字选项。
 */
void EqNetworkThread::bindAndSetSocketOptionsForWindows()
{
    socketObject.bind(MulticastPort); //绑定。
    
    socketObject.joinMulticastGroup(address); //加入组播组。
} //void EqNetworkThread::bindAndSetSocketOptionsForWindows()

#ifdef Q_OS_UNIX
/*!
 * \brief EqNetworkThread::bindAndSetSocketOptionsForUnix 针对UNIX绑定及设置套接字选项。
 */
void EqNetworkThread::bindAndSetSocketOptionsForUnix()
{
    int sockfd=0;
    
    struct ip_mreq mreq;
    
    struct sockaddr_in serv_addr;
    sockfd=socket(AF_INET, SOCK_DGRAM, 0);
    
    int flag=1;
    
    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &flag, sizeof(int));
    setsockopt(sockfd, SOL_SOCKET, SO_REUSEPORT, &flag, sizeof(int));
    
    serv_addr.sin_family=AF_INET;
    serv_addr.sin_addr.s_addr=INADDR_ANY;
    serv_addr.sin_port=htons(MulticastPort);
    
    bind(sockfd, (struct sockaddr*)&serv_addr, sizeof(sockaddr_in));

    //加入组播组：
    mreq.imr_multiaddr.s_addr=inet_addr(address.toString().toStdString().c_str());
    mreq.imr_interface.s_addr=htonl(INADDR_ANY);
    
    setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq)); //加入组播组。

    socketObject.setSocketDescriptor(sockfd);
    
} //void EqNetworkThread::bindAndSetSocketOptionsForUnix()
#endif

/*!
 * \brief EqNetworkThread::bindAndSetSocketOptions 绑定并设置套接字选项。
 */
void EqNetworkThread::bindAndSetSocketOptions()
{
#ifdef Q_OS_UNIX
    bindAndSetSocketOptionsForUnix(); //针对UNIX绑定及设置套接字选项。
#else
    bindAndSetSocketOptionsForWindows(); //针对Windows绑定及设置字选项。
#endif
} //void EqNetworkThread::bindAndSetSocketOptions()

/*!
 * \brief EqNetworkThread::castData 发送数据。
 * \param dataToCast 要发送的数据。
 */
void EqNetworkThread::castData(string dataToCast)
{
    std::string msg=dataToCast; //消息内容。
    
    QHostAddress sa("239.173.40.5");
    
    QByteArray dataByteArray=QByteArray::fromStdString(dataToCast); //转换成字节数组。
    
    dgs.writeDatagram(dataByteArray, sa, MulticastPort); //发送数据包。
} //void EqNetworkThread::castData(QByteArray dataToCast)
