﻿#include "EqPreCompile.h" //QApplication

#include "ServicePublisher.h"


/*!
 * \brief GameInfo::GameInfo 默认构造函数。
 */
ServicePublisher::ServicePublisher()
{
    connectSignals(); //连接信号槽。

    listen(); //监听。加入组播组。
} //ServicePublisher::ServicePublisher()

/*!
 * \brief Communicator::quit 退出。
 */
void ServicePublisher::quit()
{
    networkThread.quit(); //工作线程退出。
    networkThread.wait(); //等待线程真正退出。
} //void Communicator::quit()

/*!
 * \brief ServicePublisher::publishService 发布服务。
 * \param LanServiceName 服务名字。
 * \param LanServicePort 服务端口。
 * \param LanServiceProtocolType 服务协议类型。
 */
void ServicePublisher::publishService(QString LanServiceName,quint16 LanServicePort,exist::ServicePublishMessage::ServiceProtocolType LanServiceProtocolType)
{
    Q_UNUSED(LanServiceName); //不使用这个参数。
    Q_UNUSED(LanServicePort); //不使用这个参数。
    Q_UNUSED(LanServiceProtocolType); //不使用这个参数。

    exist::ExistMessage existMessage; //消息。

    existMessage.set_messagetype (exist::ExistMessage::SERVICEPUBLISH); //设置消息类型。

    exist::ServicePublishMessage * servicePublishMessage=existMessage.mutable_servicepublishmessage (); //获取服务发布消息对象。

    servicePublishMessage->set_name (LanServiceName.toStdString ()); //设置服务名字。
    servicePublishMessage->set_port (LanServicePort); //设置端口号。
    servicePublishMessage->set_protocoltype (LanServiceProtocolType); //设置协议类型。

    string packageString=existMessage.SerializeAsString (); //序列化为字符串。

    emit shoudCastData(packageString); //发射信号，应当发送数据。
} //void ServicePublisher::publishService(QString LanServiceName,quint16 LanServicePort,ServiceProtocolType LanServiceProtocolType)

/*!
 * \brief UdpServer::listen 监听。
 */
void ServicePublisher::listen()
{
    startNetworkThread(); //启动网络线程。
} //void UdpServer::listen()

/*!
 * \brief ServicePublisher::~ServicePublisher 析构函数。
 */
ServicePublisher::~ServicePublisher()
{
} //ServicePublisher::~ServicePublisher()

/*!
 * \brief ServicePublisher::connectSignals 连接信号槽。
 */
void ServicePublisher::connectSignals()
{
    connect (&networkThread,&EqNetworkThread::readedSocketData,this,&ServicePublisher::processDatagram); //读取到了数据，则处理。
    connect (this,&ServicePublisher::shoudCastData,&networkThread,&EqNetworkThread::castData); //应当发送数据，则发送。
} //void ServicePublisher::connectSignals()

/*!
 * \brief UdpServer::processDatagram 处理数据包。
 * \param datagram 数据包。
 */
void ServicePublisher::processDatagram(char *  datagram, int datagramLength, QHostAddress senderAddress)
{
    ExistMessage existMessage;

    if (datagramLength>0) //长度有效。
    {
      string datagramtoStdString(datagram,datagramLength); //数据包。

      existMessage.ParseFromString (datagramtoStdString); //解析数据包。

      switch (existMessage.messagetype ()) //根据消息类型来处理。
      {
      case ExistMessage::SERVICEPUBLISH: //服务发布。
          reportServiceDiscovered(existMessage,senderAddress.toString().toStdString()); //报告，发现了服务。

          break; //跳出。

      default: //默认情况。
          break; //跳出。
      } //switch (existMessage.messagetype ()) //根据消息类型来处理。

    } //if (datagramLength>0) //长度有效。
} //void UdpServer::processDatagram(QByteArray datagram)

/*!
 * \brief ServicePublisher::reportServiceDiscovered 报告，发现了服务。
 * \param existMessage 服务消息数据包。
 */
void ServicePublisher::reportServiceDiscovered(ExistMessage existMessage,string address)
{
    const ServicePublishMessage  servicePublishMessage=existMessage.servicepublishmessage (); //获取消息发布消息对象。

    QString serviceName=QString::fromStdString (servicePublishMessage.name ()); //获取名字。
    quint16 servicePort=servicePublishMessage.port (); //获取端口号。
    ServicePublishMessage::ServiceProtocolType serviceProtocolType=servicePublishMessage.protocoltype (); //获取协议类型。

    emit serviceDiscovered(serviceName,servicePort,serviceProtocolType,address); //发射信号，发现了服务。
} //void ServicePublisher::reportServiceDiscovered(ExistMessage existMessage)

/*!
 * \brief ServicePublisher::startNetworkThread 启动网络线程。
 */
void ServicePublisher::startNetworkThread()
{
    networkThread.start(); //启动线程。
} //void ServicePublisher::startNetworkThread()
